@@ -37,7 +37,7 @@ has 'async'            => is => 'ro',   isa => Bool, default => FALSE;
 
 has 'close_all_files'  => is => 'ro',   isa => Bool, default => FALSE;
 
-has 'cmd'              => is => 'ro',   isa => ArrayRef | NonEmptySimpleStr,
+has 'cmd'              => is => 'ro',   isa => ArrayRef | Str,
    required            => TRUE;
 
 has 'detach'           => is => 'ro',   isa => Bool, default => FALSE;
@@ -1023,7 +1023,7 @@ Peter Flanigan, C<< <pjfl@cpan.org> >>
 
 =head1 License and Copyright
 
-Copyright (c) 2016 Peter Flanigan. All rights reserved
+Copyright (c) 2017 Peter Flanigan. All rights reserved
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself. See L<perlartistic>
